/**
 * @file main.c
 * @brief Main routine
 *
 * @section License
 *
 * Copyright (C) 2010-2016 Oryx Embedded SARL. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 1.7.0
 **/

//Dependencies
#include "soc_am335x.h"
#include "hw_control_am335x.h"
#include "cp15.h"
#include "cache.h"
#include "mmu.h"
#include "interrupt.h"
#include "gpio_v2.h"
#include "dmtimer.h"
#include "watchdog.h"
#include "evmskam335x.h"
#include "core/net.h"
#include "drivers/am335x_eth.h"
#include "drivers/ar8031.h"
#include "dhcp/dhcp_client.h"
#include "ipv6/slaac.h"
#include "snmp/snmp_agent.h"
#include "snmp/mib2_module.h"
#include "snmp/mib2_impl.h"
#include "private_mib_module.h"
#include "private_mib_impl.h"
#include "debug.h"

//Application configuration
#define APP_IF0_MAC_ADDR "00-00-00-00-00-00"

#define APP_IF0_USE_DHCP ENABLED
#define APP_IF0_IPV4_HOST_ADDR "192.168.0.20"
#define APP_IF0_IPV4_SUBNET_MASK "255.255.255.0"
#define APP_IF0_IPV4_DEFAULT_GATEWAY "192.168.0.254"
#define APP_IF0_IPV4_PRIMARY_DNS "8.8.8.8"
#define APP_IF0_IPV4_SECONDARY_DNS "8.8.4.4"

#define APP_IF0_USE_SLAAC ENABLED
#define APP_IF0_IPV6_LINK_LOCAL_ADDR "fe80::3358"
#define APP_IF0_IPV6_PREFIX "2001:db8::"
#define APP_IF0_IPV6_PREFIX_LENGTH 64
#define APP_IF0_IPV6_GLOBAL_ADDR "2001:db8::3358"
#define APP_IF0_IPV6_ROUTER "fe80::1"
#define APP_IF0_IPV6_PRIMARY_DNS "2001:4860:4860::8888"
#define APP_IF0_IPV6_SECONDARY_DNS "2001:4860:4860::8844"

#define APP_IF1_MAC_ADDR "00-00-00-00-00-00"

#define APP_IF1_USE_DHCP ENABLED
#define APP_IF1_IPV4_HOST_ADDR "192.168.0.21"
#define APP_IF1_IPV4_SUBNET_MASK "255.255.255.0"
#define APP_IF1_IPV4_DEFAULT_GATEWAY "192.168.0.254"
#define APP_IF1_IPV4_PRIMARY_DNS "8.8.8.8"
#define APP_IF1_IPV4_SECONDARY_DNS "8.8.4.4"

#define APP_IF1_USE_SLAAC ENABLED
#define APP_IF1_IPV6_LINK_LOCAL_ADDR "fe80::3359"
#define APP_IF1_IPV6_PREFIX "2001:db8::"
#define APP_IF1_IPV6_PREFIX_LENGTH 64
#define APP_IF1_IPV6_GLOBAL_ADDR "2001:db8::3359"
#define APP_IF1_IPV6_ROUTER "fe80::1"
#define APP_IF1_IPV6_PRIMARY_DNS "2001:4860:4860::8888"
#define APP_IF1_IPV6_SECONDARY_DNS "2001:4860:4860::8844"

//DMTimer2 input clock frequency
#define DMTIMER2_INPUT_CLK 24000000
//DMTimer2 tick frequency
#define DMTIMER2_TICK_FREQ 1000
//DMTimer2 reload value
#define DMTIMER2_RELOAD_VALUE (0xFFFFFFFF - (DMTIMER2_INPUT_CLK / DMTIMER2_TICK_FREQ))

//DDR memory region
#define DDR_START_ADDR 0x80000000
#define DDR_NUM_SECTIONS 512
//OCMC memory region
#define OCMC_START_ADDR 0x40300000
#define OCMC_NUM_SECTIONS 1
//Device memory region
#define DEV_START_ADDR 0x44000000
#define DEV_NUM_SECTIONS 960

//Page table
static volatile uint32_t pageTable[4 * 1024] __attribute__((aligned(16 * 1024)));

 //Global variables
DhcpClientSettings dhcpClientSettings;
DhcpClientCtx dhcpClientContext0;
DhcpClientCtx dhcpClientContext1;
SlaacSettings slaacSettings;
SlaacContext slaacContext0;
SlaacContext slaacContext1;
SnmpAgentSettings snmpAgentSettings;
SnmpAgentContext snmpAgentContext;


/**
 * @brief MMU configuration
 **/

void mmuInit(void)
{
   //Define DDR memory region
   REGION ddrRegion =
   {
      MMU_PGTYPE_SECTION,
      DDR_START_ADDR,
      DDR_NUM_SECTIONS,
      MMU_MEMTYPE_NORMAL_NON_SHAREABLE(MMU_CACHE_WT_NOWA, MMU_CACHE_WB_WA),
      MMU_REGION_NON_SECURE,
      MMU_AP_PRV_RW_USR_RW,
      (uint32_t *) pageTable
   };

   //OCMC memory region
   REGION ocmcRegion =
   {
      MMU_PGTYPE_SECTION,
      OCMC_START_ADDR,
      OCMC_NUM_SECTIONS,
      MMU_MEMTYPE_NORMAL_NON_SHAREABLE(MMU_CACHE_WT_NOWA, MMU_CACHE_WB_WA),
      MMU_REGION_NON_SECURE,
      MMU_AP_PRV_RW_USR_RW,
      (uint32_t *) pageTable
   };

   //Device memory region
   REGION devRegion =
   {
      MMU_PGTYPE_SECTION,
      DEV_START_ADDR,
      DEV_NUM_SECTIONS,
      MMU_MEMTYPE_DEVICE_SHAREABLE,
      MMU_REGION_NON_SECURE,
      MMU_AP_PRV_RW_USR_RW | MMU_SECTION_EXEC_NEVER,
      (uint32_t *) pageTable
   };

   //Initialize the page table
   MMUInit((uint32_t *) pageTable);

   //Map the defined regions
   MMUMemRegionMap(&ddrRegion);
   MMUMemRegionMap(&ocmcRegion);
   MMUMemRegionMap(&devRegion);

   //Enable the MMU
   MMUEnable((uint32_t *) pageTable);
}


/**
 * @brief I/O initialization
 **/

void ioInit(void)
{
   //Enable clocks for GPIO instances
   GPIO0ModuleClkConfig();
   GPIO1ModuleClkConfig();
   GPIO2ModuleClkConfig();

   //Enable GPIO modules
   GPIOModuleEnable(SOC_GPIO_0_REGS);
   GPIOModuleEnable(SOC_GPIO_1_REGS);
   GPIOModuleEnable(SOC_GPIO_2_REGS);

   //Configure LED1 as a GPIO (GPIO1_4/GPMC_AD4)
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_AD(4)) = CONTROL_CONF_MUXMODE(7);
   //Configure LED2 as a GPIO (GPIO1_5/GPMC_AD5)
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_AD(5)) = CONTROL_CONF_MUXMODE(7);
   //Configure LED3 as a GPIO (GPIO1_7/GPMC_AD7)
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_AD(7)) = CONTROL_CONF_MUXMODE(7);
   //Configure LED4 as a GPIO (GPIO1_6/GPMC_AD6)
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_AD(6)) = CONTROL_CONF_MUXMODE(7);

   //Configure LED1 as an output
   GPIODirModeSet(SOC_GPIO_1_REGS, 4, GPIO_DIR_OUTPUT);
   //Configure LED2 as an output
   GPIODirModeSet(SOC_GPIO_1_REGS, 5, GPIO_DIR_OUTPUT);
   //Configure LED3 as an output
   GPIODirModeSet(SOC_GPIO_1_REGS, 7, GPIO_DIR_OUTPUT);
   //Configure LED4 as an output
   GPIODirModeSet(SOC_GPIO_1_REGS, 6, GPIO_DIR_OUTPUT);

   //Configure SW1 as a GPIO (GPIO2_3/GPMC_OEN_REN)
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_OEN_REN) = CONTROL_CONF_RXACTIVE |
      CONTROL_CONF_MUXMODE(7);
   //Configure SW2 as a GPIO (GPIO2_2/GPMC_ADVN_ALE)
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_ADVN_ALE) = CONTROL_CONF_RXACTIVE |
      CONTROL_CONF_MUXMODE(7);
   //Configure SW3 as a GPIO (GPIO0_30/GPMC_WAIT0)
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_WAIT0) = CONTROL_CONF_RXACTIVE |
      CONTROL_CONF_MUXMODE(7);
   //Configure SW4 as a GPIO (GPIO2_5/GPMC_BE0N_CLE)
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_BE0N_CLE) =  CONTROL_CONF_RXACTIVE |
      CONTROL_CONF_MUXMODE(7);

   //Configure SW1 as an input
   GPIODirModeSet(SOC_GPIO_2_REGS, 3, GPIO_DIR_INPUT);
   //Configure SW2 as an input
   GPIODirModeSet(SOC_GPIO_2_REGS, 2, GPIO_DIR_INPUT);
   //Configure SW3 as an input
   GPIODirModeSet(SOC_GPIO_0_REGS, 30, GPIO_DIR_INPUT);
   //Configure SW4 as an input
   GPIODirModeSet(SOC_GPIO_2_REGS, 5, GPIO_DIR_INPUT);
}


/**
 * @brief DMTimer2 interrupt handler
 **/

void timer2IrqHandler(void)
{
   //Clear interrupt flag
   DMTimerIntStatusClear(SOC_DMTIMER_2_REGS, DMTIMER_INT_OVF_IT_FLAG);
   //Increment tick counter
   systemTicks++;
}


/**
 * @brief DMTimer2 initialization
 **/

void timer2Init(void)
{
   //Enable clocks for DMTimer2 instance
   DMTimer2ModuleClkConfig();

   //Disable timer
   DMTimerDisable(SOC_DMTIMER_2_REGS);
   //Disable prescaler
   DMTimerPreScalerClkDisable(SOC_DMTIMER_2_REGS);

   //Configure DMTimer2 for auto-reload and compare mode
   DMTimerModeConfigure(SOC_DMTIMER_2_REGS, DMTIMER_AUTORLD_CMP_ENABLE);
   //Set reload value
   DMTimerReloadSet(SOC_DMTIMER_2_REGS, DMTIMER2_RELOAD_VALUE);
   //Initialize counter value
   DMTimerCounterSet(SOC_DMTIMER_2_REGS, DMTIMER2_RELOAD_VALUE);

   //Enable DMTimer2 interrupts
   DMTimerIntEnable(SOC_DMTIMER_2_REGS, DMTIMER_INT_OVF_EN_FLAG);

   //Register interrupt handler
   IntRegister(SYS_INT_TINT2, timer2IrqHandler);
   //Configure interrupt priority
   IntPrioritySet(SYS_INT_TINT2, 0, AINTC_HOSTINT_ROUTE_IRQ);
   //Enable system interrupts
   IntSystemEnable(SYS_INT_TINT2);

   //Enable timer
   DMTimerEnable(SOC_DMTIMER_2_REGS);
}


/**
 * @brief Main entry point
 * @return Unused value
 **/

int_t main(void)
{
   error_t error;
   NetInterface *interface;
   MacAddr macAddr;
#if (APP_IF0_USE_DHCP == DISABLED || APP_IF1_USE_DHCP == DISABLED)
   Ipv4Addr ipv4Addr;
#endif
#if (APP_IF0_USE_SLAAC == DISABLED || APP_IF1_USE_SLAAC == DISABLED)
   Ipv6Addr ipv6Addr;
#endif
   bool_t ledState;
   systime_t ledTime;

   //Disable watchdog timer
   WatchdogTimerDisable(SOC_WDT_1_REGS);

   //Disable cache
   CacheDisable(CACHE_ALL);
   //Disable MMU
   CP15MMUDisable();

   //Configure and enable MMU
   mmuInit();
   //Enable cache
   CacheEnable(CACHE_ALL);

   //Initialize interrupt controller
   IntAINTCInit();

   //Initialize kernel
   osInitKernel();
   //Configure debug UART
   debugInit(115200);

   //Start-up message
   TRACE_INFO("\r\n");
   TRACE_INFO("**********************************\r\n");
   TRACE_INFO("*** CycloneTCP SNMP Agent Demo ***\r\n");
   TRACE_INFO("**********************************\r\n");
   TRACE_INFO("Copyright: 2010-2016 Oryx Embedded SARL\r\n");
   TRACE_INFO("Compiled: %s %s\r\n", __DATE__, __TIME__);
   TRACE_INFO("Target: AM335x\r\n");
   TRACE_INFO("\r\n");

   //Configure I/Os
   ioInit();

   //Standard MIB-II initialization
   error = mib2Init();
   //Any error to report?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to initialize MIB!\r\n");
   }

   //Private MIB initialization
   error = privateMibInit();
   //Any error to report?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to initialize MIB!\r\n");
   }

   //TCP/IP stack initialization
   error = netInit();
   //Any error to report?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to initialize TCP/IP stack!\r\n");
   }

   //Configure the first Ethernet interface
   interface = &netInterface[0];

   //Set interface name
   netSetInterfaceName(interface, "eth0");
   //Set host name
   netSetHostname(interface, "SNMPAgentDemo0");
   //Select the relevant MAC driver
   netSetDriver(interface, &am335xEthPort1Driver);
   //Select the relevant PHY driver
   netSetPhyDriver(interface, &ar8031PhyDriver);
   netSetPhyAddr(interface, 0);
   //Set host MAC address
   macStringToAddr(APP_IF0_MAC_ADDR, &macAddr);
   netSetMacAddr(interface, &macAddr);

   //Initialize network interface
   error = netConfigInterface(interface);
   //Any error to report?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to configure interface %s!\r\n", interface->name);
   }

#if (IPV4_SUPPORT == ENABLED)
#if (APP_IF0_USE_DHCP == ENABLED)
   //Get default settings
   dhcpClientGetDefaultSettings(&dhcpClientSettings);
   //Set the network interface to be configured by DHCP
   dhcpClientSettings.interface = interface;
   //Disable rapid commit option
   dhcpClientSettings.rapidCommit = FALSE;

   //DHCP client initialization
   error = dhcpClientInit(&dhcpClientContext0, &dhcpClientSettings);
   //Failed to initialize DHCP client?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to initialize DHCP client!\r\n");
   }

   //Start DHCP client
   error = dhcpClientStart(&dhcpClientContext0);
   //Failed to start DHCP client?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to start DHCP client!\r\n");
   }
#else
   //Set IPv4 host address
   ipv4StringToAddr(APP_IF0_IPV4_HOST_ADDR, &ipv4Addr);
   ipv4SetHostAddr(interface, ipv4Addr);

   //Set subnet mask
   ipv4StringToAddr(APP_IF0_IPV4_SUBNET_MASK, &ipv4Addr);
   ipv4SetSubnetMask(interface, ipv4Addr);

   //Set default gateway
   ipv4StringToAddr(APP_IF0_IPV4_DEFAULT_GATEWAY, &ipv4Addr);
   ipv4SetDefaultGateway(interface, ipv4Addr);

   //Set primary and secondary DNS servers
   ipv4StringToAddr(APP_IF0_IPV4_PRIMARY_DNS, &ipv4Addr);
   ipv4SetDnsServer(interface, 0, ipv4Addr);
   ipv4StringToAddr(APP_IF0_IPV4_SECONDARY_DNS, &ipv4Addr);
   ipv4SetDnsServer(interface, 1, ipv4Addr);
#endif
#endif

#if (IPV6_SUPPORT == ENABLED)
#if (APP_IF0_USE_SLAAC == ENABLED)
   //Get default settings
   slaacGetDefaultSettings(&slaacSettings);
   //Set the network interface to be configured
   slaacSettings.interface = interface;

   //SLAAC initialization
   error = slaacInit(&slaacContext0, &slaacSettings);
   //Failed to initialize SLAAC?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to initialize SLAAC!\r\n");
   }

   //Start IPv6 address autoconfiguration process
   error = slaacStart(&slaacContext0);
   //Failed to start SLAAC process?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to start SLAAC!\r\n");
   }
#else
   //Set link-local address
   ipv6StringToAddr(APP_IF0_IPV6_LINK_LOCAL_ADDR, &ipv6Addr);
   ipv6SetLinkLocalAddr(interface, &ipv6Addr);

   //Set IPv6 prefix
   ipv6StringToAddr(APP_IF0_IPV6_PREFIX, &ipv6Addr);
   ipv6SetPrefix(interface, 0, &ipv6Addr, APP_IF0_IPV6_PREFIX_LENGTH);

   //Set global address
   ipv6StringToAddr(APP_IF0_IPV6_GLOBAL_ADDR, &ipv6Addr);
   ipv6SetGlobalAddr(interface, 0, &ipv6Addr);

   //Set default router
   ipv6StringToAddr(APP_IF0_IPV6_ROUTER, &ipv6Addr);
   ipv6SetDefaultRouter(interface, 0, &ipv6Addr);

   //Set primary and secondary DNS servers
   ipv6StringToAddr(APP_IF0_IPV6_PRIMARY_DNS, &ipv6Addr);
   ipv6SetDnsServer(interface, 0, &ipv6Addr);
   ipv6StringToAddr(APP_IF0_IPV6_SECONDARY_DNS, &ipv6Addr);
   ipv6SetDnsServer(interface, 1, &ipv6Addr);
#endif
#endif

   //Configure the second Ethernet interface
   interface = &netInterface[1];

   //Set interface name
   netSetInterfaceName(interface, "eth1");
   //Set host name
   netSetHostname(interface, "SNMPAgentDemo1");
   //Select the relevant MAC driver
   netSetDriver(interface, &am335xEthPort2Driver);
   //Select the relevant PHY driver
   netSetPhyDriver(interface, &ar8031PhyDriver);
   netSetPhyAddr(interface, 1);
   //Set host MAC address
   macStringToAddr(APP_IF1_MAC_ADDR, &macAddr);
   netSetMacAddr(interface, &macAddr);

   //Initialize network interface
   error = netConfigInterface(interface);
   //Any error to report?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to configure interface %s!\r\n", interface->name);
   }

#if (IPV4_SUPPORT == ENABLED)
#if (APP_IF1_USE_DHCP == ENABLED)
   //Get default settings
   dhcpClientGetDefaultSettings(&dhcpClientSettings);
   //Set the network interface to be configured by DHCP
   dhcpClientSettings.interface = interface;
   //Disable rapid commit option
   dhcpClientSettings.rapidCommit = FALSE;

   //DHCP client initialization
   error = dhcpClientInit(&dhcpClientContext1, &dhcpClientSettings);
   //Failed to initialize DHCP client?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to initialize DHCP client!\r\n");
   }

   //Start DHCP client
   error = dhcpClientStart(&dhcpClientContext1);
   //Failed to start DHCP client?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to start DHCP client!\r\n");
   }
#else
   //Set IPv4 host address
   ipv4StringToAddr(APP_IF1_IPV4_HOST_ADDR, &ipv4Addr);
   ipv4SetHostAddr(interface, ipv4Addr);

   //Set subnet mask
   ipv4StringToAddr(APP_IF1_IPV4_SUBNET_MASK, &ipv4Addr);
   ipv4SetSubnetMask(interface, ipv4Addr);

   //Set default gateway
   ipv4StringToAddr(APP_IF1_IPV4_DEFAULT_GATEWAY, &ipv4Addr);
   ipv4SetDefaultGateway(interface, ipv4Addr);

   //Set primary and secondary DNS servers
   ipv4StringToAddr(APP_IF1_IPV4_PRIMARY_DNS, &ipv4Addr);
   ipv4SetDnsServer(interface, 0, ipv4Addr);
   ipv4StringToAddr(APP_IF1_IPV4_SECONDARY_DNS, &ipv4Addr);
   ipv4SetDnsServer(interface, 1, ipv4Addr);
#endif
#endif

#if (IPV6_SUPPORT == ENABLED)
#if (APP_IF1_USE_SLAAC == ENABLED)
   //Get default settings
   slaacGetDefaultSettings(&slaacSettings);
   //Set the network interface to be configured
   slaacSettings.interface = interface;

   //SLAAC initialization
   error = slaacInit(&slaacContext1, &slaacSettings);
   //Failed to initialize SLAAC?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to initialize SLAAC!\r\n");
   }

   //Start IPv6 address autoconfiguration process
   error = slaacStart(&slaacContext1);
   //Failed to start SLAAC process?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to start SLAAC!\r\n");
   }
#else
   //Set link-local address
   ipv6StringToAddr(APP_IF1_IPV6_LINK_LOCAL_ADDR, &ipv6Addr);
   ipv6SetLinkLocalAddr(interface, &ipv6Addr);

   //Set IPv6 prefix
   ipv6StringToAddr(APP_IF1_IPV6_PREFIX, &ipv6Addr);
   ipv6SetPrefix(interface, 0, &ipv6Addr, APP_IF1_IPV6_PREFIX_LENGTH);

   //Set global address
   ipv6StringToAddr(APP_IF1_IPV6_GLOBAL_ADDR, &ipv6Addr);
   ipv6SetGlobalAddr(interface, 0, &ipv6Addr);

   //Set default router
   ipv6StringToAddr(APP_IF1_IPV6_ROUTER, &ipv6Addr);
   ipv6SetDefaultRouter(interface, 0, &ipv6Addr);

   //Set primary and secondary DNS servers
   ipv6StringToAddr(APP_IF1_IPV6_PRIMARY_DNS, &ipv6Addr);
   ipv6SetDnsServer(interface, 0, &ipv6Addr);
   ipv6StringToAddr(APP_IF1_IPV6_SECONDARY_DNS, &ipv6Addr);
   ipv6SetDnsServer(interface, 1, &ipv6Addr);
#endif
#endif

   //Get default settings
   snmpAgentGetDefaultSettings(&snmpAgentSettings);
   //Bind SNMP agent to both network interfaces
   snmpAgentSettings.interface = NULL;
   //SNMP version identifier
   snmpAgentSettings.version = SNMP_VERSION_2;
   //Read-only community string
   strcpy(snmpAgentSettings.readOnlyCommunity, "public");
   //Read-write community string
   strcpy(snmpAgentSettings.readWriteCommunity, "private");
   //Trap community string
   strcpy(snmpAgentSettings.trapCommunity, "public");

   //SNMP agent initialization
   error = snmpAgentInit(&snmpAgentContext, &snmpAgentSettings);
   //Failed to initialize SNMP agent?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to initialize SNMP agent!\r\n");
   }

   //Load standard MIB-II
   error = snmpAgentLoadMib(&snmpAgentContext, &mib2Module);
   //Failed to load MIB?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to load MIB!\r\n");
   }

   //Load private MIB
   error = snmpAgentLoadMib(&snmpAgentContext, &privateMibModule);
   //Failed to load MIB?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to load MIB!\r\n");
   }

   //Start SNMP agent
   error = snmpAgentStart(&snmpAgentContext);
   //Failed to start SNMP agent?
   if(error)
   {
      //Debug message
      TRACE_ERROR("Failed to start SNMP agent!\r\n");
   }

   //Initialize DMTimer2
   timer2Init();

   //Enable processor IRQ
   IntMasterIRQEnable();

   //Initialize variables
   ledState = 0;
   ledTime = 0;

   //Endless loop
   while(1)
   {
      //Handle TCP/IP events
      netTask();
      //Handle SNMP agent events
      snmpAgentTask(&snmpAgentContext);

      //LED task
      if((int32_t)(systemTicks - ledTime) > 0)
      {
         if(ledState == 0)
         {
            GPIOPinWrite(SOC_GPIO_1_REGS, 6, GPIO_PIN_HIGH);
            ledState = 1;
            ledTime = systemTicks + 100;
         }
         else
         {
            GPIOPinWrite(SOC_GPIO_1_REGS, 6, GPIO_PIN_LOW);
            ledState = 0;
            ledTime = systemTicks + 900;
         }
      }
   }
}
